# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt
from typing import Optional
from typing_extensions import Annotated
from postfinancecheckout.models.user_account_role import UserAccountRole
from postfinancecheckout.models.user_account_role_list_response import UserAccountRoleListResponse
from postfinancecheckout.models.user_space_role import UserSpaceRole
from postfinancecheckout.models.user_space_role_list_response import UserSpaceRoleListResponse

from postfinancecheckout.api_client import ApiClient, RequestSerialized
from postfinancecheckout.api_response import ApiResponse
from postfinancecheckout.rest import RESTResponseType



class ApplicationUsersRolesService:
    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)


    @validate_call
    def delete_application_users_user_id_account_roles(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Unassign a role from an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_application_users_user_id_account_roles_with_http_info(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Unassign a role from an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_application_users_user_id_account_roles_without_preload_content(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Unassign a role from an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _delete_application_users_user_id_account_roles_serialize(
        self,
        user_id,
        role_id,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if user_id is not None:
            _path_params['userId'] = user_id
        # process the query parameters
        if role_id is not None:
            
            _query_params.append(('roleId', role_id))
            
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/application-users/{userId}/account-roles',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def delete_application_users_user_id_space_roles(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Unassign a role from an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_application_users_user_id_space_roles_with_http_info(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Unassign a role from an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_application_users_user_id_space_roles_without_preload_content(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Unassign a role from an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _delete_application_users_user_id_space_roles_serialize(
        self,
        user_id,
        role_id,
        space,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if user_id is not None:
            _path_params['userId'] = user_id
        # process the query parameters
        if role_id is not None:
            
            _query_params.append(('roleId', role_id))
            
        # process the header parameters
        if space is not None:
            _header_params['Space'] = space
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/application-users/{userId}/space-roles',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_application_users_user_id_account_roles(
        self,
        user_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserAccountRoleListResponse:
        """List all roles of an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserAccountRoleListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_application_users_user_id_account_roles_with_http_info(
        self,
        user_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserAccountRoleListResponse]:
        """List all roles of an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserAccountRoleListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_application_users_user_id_account_roles_without_preload_content(
        self,
        user_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List all roles of an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserAccountRoleListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_application_users_user_id_account_roles_serialize(
        self,
        user_id,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if user_id is not None:
            _path_params['userId'] = user_id
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/application-users/{userId}/account-roles',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_application_users_user_id_space_roles(
        self,
        user_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserSpaceRoleListResponse:
        """List all roles of an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserSpaceRoleListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_application_users_user_id_space_roles_with_http_info(
        self,
        user_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserSpaceRoleListResponse]:
        """List all roles of an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserSpaceRoleListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_application_users_user_id_space_roles_without_preload_content(
        self,
        user_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List all roles of an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserSpaceRoleListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_application_users_user_id_space_roles_serialize(
        self,
        user_id,
        space,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if user_id is not None:
            _path_params['userId'] = user_id
        # process the query parameters
        # process the header parameters
        if space is not None:
            _header_params['Space'] = space
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/application-users/{userId}/space-roles',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def post_application_users_user_id_account_roles(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        applies_on_sub_account: Annotated[Optional[StrictBool], Field(description="Whether the role should be assigned to the user in subaccounts only.")] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserAccountRole:
        """Assign a role to an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param applies_on_sub_account: Whether the role should be assigned to the user in subaccounts only.
        :type applies_on_sub_account: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            account=account,
            applies_on_sub_account=applies_on_sub_account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserAccountRole",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_application_users_user_id_account_roles_with_http_info(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        applies_on_sub_account: Annotated[Optional[StrictBool], Field(description="Whether the role should be assigned to the user in subaccounts only.")] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserAccountRole]:
        """Assign a role to an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param applies_on_sub_account: Whether the role should be assigned to the user in subaccounts only.
        :type applies_on_sub_account: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            account=account,
            applies_on_sub_account=applies_on_sub_account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserAccountRole",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_application_users_user_id_account_roles_without_preload_content(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        applies_on_sub_account: Annotated[Optional[StrictBool], Field(description="Whether the role should be assigned to the user in subaccounts only.")] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Assign a role to an application user for an account



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param applies_on_sub_account: Whether the role should be assigned to the user in subaccounts only.
        :type applies_on_sub_account: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_application_users_user_id_account_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            account=account,
            applies_on_sub_account=applies_on_sub_account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserAccountRole",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _post_application_users_user_id_account_roles_serialize(
        self,
        user_id,
        role_id,
        account,
        applies_on_sub_account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if user_id is not None:
            _path_params['userId'] = user_id
        # process the query parameters
        if role_id is not None:
            
            _query_params.append(('roleId', role_id))
            
        if applies_on_sub_account is not None:
            
            _query_params.append(('appliesOnSubAccount', applies_on_sub_account))
            
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='POST',
            resource_path='/application-users/{userId}/account-roles',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def post_application_users_user_id_space_roles(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserSpaceRole:
        """Assign a role to an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserSpaceRole",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_application_users_user_id_space_roles_with_http_info(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserSpaceRole]:
        """Assign a role to an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserSpaceRole",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_application_users_user_id_space_roles_without_preload_content(
        self,
        user_id: StrictInt,
        role_id: StrictInt,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Assign a role to an application user for a space



        :param user_id: (required)
        :type user_id: int
        :param role_id: (required)
        :type role_id: int
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_application_users_user_id_space_roles_serialize(
            user_id=user_id,
            role_id=role_id,
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserSpaceRole",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _post_application_users_user_id_space_roles_serialize(
        self,
        user_id,
        role_id,
        space,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if user_id is not None:
            _path_params['userId'] = user_id
        # process the query parameters
        if role_id is not None:
            
            _query_params.append(('roleId', role_id))
            
        # process the header parameters
        if space is not None:
            _header_params['Space'] = space
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='POST',
            resource_path='/application-users/{userId}/space-roles',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )






# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.address_create import AddressCreate
from postfinancecheckout.models.line_item_create import LineItemCreate
from postfinancecheckout.models.tokenization_mode import TokenizationMode
from postfinancecheckout.models.transaction_completion_behavior import TransactionCompletionBehavior
from typing import Optional, Set
from typing_extensions import Self

class AbstractTransactionPending(BaseModel):
    """
    AbstractTransactionPending
    """ # noqa: E501
    customer_email_address: Optional[Annotated[str, Field(strict=True, max_length=254)]] = Field(default=None, description="The customer's email address.", alias="customerEmailAddress")
    shipping_method: Optional[Annotated[str, Field(strict=True, max_length=200)]] = Field(default=None, description="The name of the shipping method used to ship the products.", alias="shippingMethod")
    invoice_merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the invoice.", alias="invoiceMerchantReference")
    success_url: Optional[Annotated[str, Field(min_length=9, strict=True, max_length=2000)]] = Field(default=None, description="The URL to redirect the customer back to after they successfully authenticated their payment.", alias="successUrl")
    time_zone: Optional[StrictStr] = Field(default=None, description="The customer's time zone, which affects how dates and times are formatted when communicating with the customer.", alias="timeZone")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    tokenization_mode: Optional[TokenizationMode] = Field(default=None, alias="tokenizationMode")
    allowed_payment_method_brands: Optional[List[StrictInt]] = Field(default=None, description="The payment method brands that can be used to authorize the transaction.", alias="allowedPaymentMethodBrands")
    completion_behavior: Optional[TransactionCompletionBehavior] = Field(default=None, alias="completionBehavior")
    token: Optional[StrictInt] = Field(default=None, description="The payment token that should be used to charge the customer.")
    line_items: Optional[List[LineItemCreate]] = Field(default=None, description="The line items purchased by the customer.", alias="lineItems")
    meta_data: Optional[Dict[str, StrictStr]] = Field(default=None, description="Allow to store additional information about the object.", alias="metaData")
    customer_id: Optional[StrictStr] = Field(default=None, description="The unique identifier of the customer in the external system.", alias="customerId")
    shipping_address: Optional[AddressCreate] = Field(default=None, alias="shippingAddress")
    currency: Optional[StrictStr] = Field(default=None, description="The three-letter code (ISO 4217 format) of the transaction's currency.")
    billing_address: Optional[AddressCreate] = Field(default=None, alias="billingAddress")
    merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the transaction.", alias="merchantReference")
    allowed_payment_method_configurations: Optional[List[StrictInt]] = Field(default=None, description="The payment method configurations that can be used to authorize the transaction.", alias="allowedPaymentMethodConfigurations")
    failed_url: Optional[Annotated[str, Field(min_length=9, strict=True, max_length=2000)]] = Field(default=None, description="The URL to redirect the customer back to after they canceled or failed to authenticated their payment.", alias="failedUrl")
    __properties: ClassVar[List[str]] = ["customerEmailAddress", "shippingMethod", "invoiceMerchantReference", "successUrl", "timeZone", "language", "tokenizationMode", "allowedPaymentMethodBrands", "completionBehavior", "token", "lineItems", "metaData", "customerId", "shippingAddress", "currency", "billingAddress", "merchantReference", "allowedPaymentMethodConfigurations", "failedUrl"]

    @field_validator('invoice_merchant_reference')
    def invoice_merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    @field_validator('merchant_reference')
    def merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AbstractTransactionPending from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of shipping_address
        if self.shipping_address:
            _dict['shippingAddress'] = self.shipping_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of billing_address
        if self.billing_address:
            _dict['billingAddress'] = self.billing_address.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AbstractTransactionPending from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "customerEmailAddress": obj.get("customerEmailAddress"),
            "shippingMethod": obj.get("shippingMethod"),
            "invoiceMerchantReference": obj.get("invoiceMerchantReference"),
            "successUrl": obj.get("successUrl"),
            "timeZone": obj.get("timeZone"),
            "language": obj.get("language"),
            "tokenizationMode": obj.get("tokenizationMode"),
            "allowedPaymentMethodBrands": obj.get("allowedPaymentMethodBrands"),
            "completionBehavior": obj.get("completionBehavior"),
            "token": obj.get("token"),
            "lineItems": [LineItemCreate.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "metaData": obj.get("metaData"),
            "customerId": obj.get("customerId"),
            "shippingAddress": AddressCreate.from_dict(obj["shippingAddress"]) if obj.get("shippingAddress") is not None else None,
            "currency": obj.get("currency"),
            "billingAddress": AddressCreate.from_dict(obj["billingAddress"]) if obj.get("billingAddress") is not None else None,
            "merchantReference": obj.get("merchantReference"),
            "allowedPaymentMethodConfigurations": obj.get("allowedPaymentMethodConfigurations"),
            "failedUrl": obj.get("failedUrl")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.creation_entity_state import CreationEntityState
from postfinancecheckout.models.document_template import DocumentTemplate
from postfinancecheckout.models.dunning_flow import DunningFlow
from typing import Optional, Set
from typing_extensions import Self

class DunningFlowLevel(BaseModel):
    """
    DunningFlowLevel
    """ # noqa: E501
    period: Optional[StrictStr] = Field(default=None, description="The duration of the level before switching to the next one.")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    reminder_template: Optional[DocumentTemplate] = Field(default=None, alias="reminderTemplate")
    priority: Optional[StrictInt] = Field(default=None, description="The priority indicates the sort order of the level. A low value indicates that the level is executed before any level with a higher value. Any change to this value affects future level selections. The value has to pe unique per dunning flow.")
    title: Optional[Dict[str, StrictStr]] = Field(default=None, description="The title is used to communicate the dunning level to the customer within the reminder.")
    processor: Optional[StrictInt] = None
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    document_text: Optional[Dict[str, StrictStr]] = Field(default=None, description="This text is put in the reminder document of this dunning flow level.", alias="documentText")
    name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The dunning flow level name is used internally to identify the dunning flow level. For example the name is used within search fields and hence it should be distinct and descriptive.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[CreationEntityState] = None
    flow: Optional[DunningFlow] = None
    __properties: ClassVar[List[str]] = ["period", "plannedPurgeDate", "reminderTemplate", "priority", "title", "processor", "version", "linkedSpaceId", "documentText", "name", "id", "state", "flow"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DunningFlowLevel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "period",
            "planned_purge_date",
            "priority",
            "title",
            "processor",
            "version",
            "linked_space_id",
            "document_text",
            "name",
            "id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of reminder_template
        if self.reminder_template:
            _dict['reminderTemplate'] = self.reminder_template.to_dict()
        # override the default output from pydantic by calling `to_dict()` of flow
        if self.flow:
            _dict['flow'] = self.flow.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DunningFlowLevel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "period": obj.get("period"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "reminderTemplate": DocumentTemplate.from_dict(obj["reminderTemplate"]) if obj.get("reminderTemplate") is not None else None,
            "priority": obj.get("priority"),
            "title": obj.get("title"),
            "processor": obj.get("processor"),
            "version": obj.get("version"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "documentText": obj.get("documentText"),
            "name": obj.get("name"),
            "id": obj.get("id"),
            "state": obj.get("state"),
            "flow": DunningFlow.from_dict(obj["flow"]) if obj.get("flow") is not None else None
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class AnalyticsQueryExecutionRequest(BaseModel):
    """
    AnalyticsQueryExecutionRequest
    """ # noqa: E501
    sftp_dispatch_settings_id: Optional[StrictInt] = Field(default=None, description="Optional. ID of the active SFTP configuration to use (associated with the target account). This is only required if the result file is scheduled for delivery to a remote SFTP server.", alias="sftpDispatchSettingsId")
    sftp_dispatch_result_file_rename_pattern: Optional[StrictStr] = Field(default=None, description="Optional. Renaming pattern used for the result file during SFTP delivery. You can use a combination of fixed Latin text and timestamp variables (e.g., \"transaction_report_{YYYMMDD_hhmmss}\"). Supported variable formats: DDMMYY, MMDDYY, YYYYMMDD, DD_MM_YY, DD-MM-YY, YYYY-MM-DD, YYYY_MM_DD, YYYYMMDD_hhmmss, YYYY-MM-DD_hh-mm-ss.", alias="sftpDispatchResultFileRenamePattern")
    sql: Optional[Annotated[str, Field(min_length=3, strict=True, max_length=100000)]] = Field(default=None, description="The SQL query (in PrestoDB dialect) to execute on the analytics database. This query defines exactly which data should be retrieved.")
    __properties: ClassVar[List[str]] = ["sftpDispatchSettingsId", "sftpDispatchResultFileRenamePattern", "sql"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AnalyticsQueryExecutionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AnalyticsQueryExecutionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "sftpDispatchSettingsId": obj.get("sftpDispatchSettingsId"),
            "sftpDispatchResultFileRenamePattern": obj.get("sftpDispatchResultFileRenamePattern"),
            "sql": obj.get("sql")
        })
        return _obj





# flake8: noqa

# import apis into api package
from postfinancecheckout.service.accounts_service import AccountsService
from postfinancecheckout.service.analytics_queries_service import AnalyticsQueriesService
from postfinancecheckout.service.application_users_service import ApplicationUsersService
from postfinancecheckout.service.application_users_roles_service import ApplicationUsersRolesService
from postfinancecheckout.service.bank_accounts_service import BankAccountsService
from postfinancecheckout.service.bank_transactions_service import BankTransactionsService
from postfinancecheckout.service.charge_attempts_service import ChargeAttemptsService
from postfinancecheckout.service.charge_bank_transactions_service import ChargeBankTransactionsService
from postfinancecheckout.service.charge_flow_levels_service import ChargeFlowLevelsService
from postfinancecheckout.service.charge_flows_service import ChargeFlowsService
from postfinancecheckout.service.charge_flows_level_payment_links_service import ChargeFlowsLevelPaymentLinksService
from postfinancecheckout.service.condition_types_service import ConditionTypesService
from postfinancecheckout.service.countries_service import CountriesService
from postfinancecheckout.service.currencies_service import CurrenciesService
from postfinancecheckout.service.currency_bank_accounts_service import CurrencyBankAccountsService
from postfinancecheckout.service.customer_addresses_service import CustomerAddressesService
from postfinancecheckout.service.customer_comments_service import CustomerCommentsService
from postfinancecheckout.service.customers_service import CustomersService
from postfinancecheckout.service.delivery_indications_service import DeliveryIndicationsService
from postfinancecheckout.service.document_template_types_service import DocumentTemplateTypesService
from postfinancecheckout.service.document_templates_service import DocumentTemplatesService
from postfinancecheckout.service.dunning_cases_service import DunningCasesService
from postfinancecheckout.service.dunning_flow_levels_service import DunningFlowLevelsService
from postfinancecheckout.service.dunning_flows_service import DunningFlowsService
from postfinancecheckout.service.express_checkout_service import ExpressCheckoutService
from postfinancecheckout.service.external_transfer_bank_transactions_service import ExternalTransferBankTransactionsService
from postfinancecheckout.service.human_users_service import HumanUsersService
from postfinancecheckout.service.human_users_roles_service import HumanUsersRolesService
from postfinancecheckout.service.internal_transfer_bank_transactions_service import InternalTransferBankTransactionsService
from postfinancecheckout.service.label_descriptors_service import LabelDescriptorsService
from postfinancecheckout.service.languages_service import LanguagesService
from postfinancecheckout.service.legal_organization_forms_service import LegalOrganizationFormsService
from postfinancecheckout.service.manual_tasks_service import ManualTasksService
from postfinancecheckout.service.payment_connector_configurations_service import PaymentConnectorConfigurationsService
from postfinancecheckout.service.payment_connectors_service import PaymentConnectorsService
from postfinancecheckout.service.payment_links_service import PaymentLinksService
from postfinancecheckout.service.payment_method_brands_service import PaymentMethodBrandsService
from postfinancecheckout.service.payment_method_configurations_service import PaymentMethodConfigurationsService
from postfinancecheckout.service.payment_methods_service import PaymentMethodsService
from postfinancecheckout.service.payment_processor_configurations_service import PaymentProcessorConfigurationsService
from postfinancecheckout.service.payment_processors_service import PaymentProcessorsService
from postfinancecheckout.service.payment_sales_channels_service import PaymentSalesChannelsService
from postfinancecheckout.service.payment_terminal_transaction_summaries_service import PaymentTerminalTransactionSummariesService
from postfinancecheckout.service.payment_terminals_service import PaymentTerminalsService
from postfinancecheckout.service.permissions_service import PermissionsService
from postfinancecheckout.service.refund_bank_transactions_service import RefundBankTransactionsService
from postfinancecheckout.service.refund_comments_service import RefundCommentsService
from postfinancecheckout.service.refund_recovery_bank_transactions_service import RefundRecoveryBankTransactionsService
from postfinancecheckout.service.refunds_service import RefundsService
from postfinancecheckout.service.roles_service import RolesService
from postfinancecheckout.service.single_sign_on_users_service import SingleSignOnUsersService
from postfinancecheckout.service.single_sign_on_users_roles_service import SingleSignOnUsersRolesService
from postfinancecheckout.service.spaces_service import SpacesService
from postfinancecheckout.service.static_values_service import StaticValuesService
from postfinancecheckout.service.subscribers_service import SubscribersService
from postfinancecheckout.service.subscription_affiliates_service import SubscriptionAffiliatesService
from postfinancecheckout.service.subscription_charges_service import SubscriptionChargesService
from postfinancecheckout.service.subscription_ledger_entries_service import SubscriptionLedgerEntriesService
from postfinancecheckout.service.subscription_metric_usage_reports_service import SubscriptionMetricUsageReportsService
from postfinancecheckout.service.subscription_metrics_service import SubscriptionMetricsService
from postfinancecheckout.service.subscription_period_bills_service import SubscriptionPeriodBillsService
from postfinancecheckout.service.subscription_product_component_groups_service import SubscriptionProductComponentGroupsService
from postfinancecheckout.service.subscription_product_components_service import SubscriptionProductComponentsService
from postfinancecheckout.service.subscription_product_metered_fee_tiers_service import SubscriptionProductMeteredFeeTiersService
from postfinancecheckout.service.subscription_product_metered_fees_service import SubscriptionProductMeteredFeesService
from postfinancecheckout.service.subscription_product_period_fees_service import SubscriptionProductPeriodFeesService
from postfinancecheckout.service.subscription_product_retirements_service import SubscriptionProductRetirementsService
from postfinancecheckout.service.subscription_product_setup_fees_service import SubscriptionProductSetupFeesService
from postfinancecheckout.service.subscription_product_version_retirements_service import SubscriptionProductVersionRetirementsService
from postfinancecheckout.service.subscription_product_versions_service import SubscriptionProductVersionsService
from postfinancecheckout.service.subscription_products_service import SubscriptionProductsService
from postfinancecheckout.service.subscription_suspensions_service import SubscriptionSuspensionsService
from postfinancecheckout.service.subscription_versions_service import SubscriptionVersionsService
from postfinancecheckout.service.subscriptions_service import SubscriptionsService
from postfinancecheckout.service.token_versions_service import TokenVersionsService
from postfinancecheckout.service.tokens_service import TokensService
from postfinancecheckout.service.transaction_client_platforms_service import TransactionClientPlatformsService
from postfinancecheckout.service.transaction_comments_service import TransactionCommentsService
from postfinancecheckout.service.transaction_completions_service import TransactionCompletionsService
from postfinancecheckout.service.transaction_invoice_comments_service import TransactionInvoiceCommentsService
from postfinancecheckout.service.transaction_invoices_service import TransactionInvoicesService
from postfinancecheckout.service.transaction_line_item_versions_service import TransactionLineItemVersionsService
from postfinancecheckout.service.transaction_voids_service import TransactionVoidsService
from postfinancecheckout.service.transactions_service import TransactionsService
from postfinancecheckout.service.web_apps_service import WebAppsService
from postfinancecheckout.service.webhook_encryption_keys_service import WebhookEncryptionKeysService
from postfinancecheckout.service.webhook_listeners_service import WebhookListenersService
from postfinancecheckout.service.webhook_urls_service import WebhookURLsService




# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from postfinancecheckout.models.address import Address
from postfinancecheckout.models.charge_attempt_environment import ChargeAttemptEnvironment
from postfinancecheckout.models.customers_presence import CustomersPresence
from postfinancecheckout.models.environment import Environment
from postfinancecheckout.models.failure_reason import FailureReason
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.payment_connector_configuration import PaymentConnectorConfiguration
from postfinancecheckout.models.payment_terminal import PaymentTerminal
from postfinancecheckout.models.token import Token
from postfinancecheckout.models.tokenization_mode import TokenizationMode
from postfinancecheckout.models.transaction_completion_behavior import TransactionCompletionBehavior
from postfinancecheckout.models.transaction_environment_selection_strategy import TransactionEnvironmentSelectionStrategy
from postfinancecheckout.models.transaction_group import TransactionGroup
from postfinancecheckout.models.transaction_state import TransactionState
from postfinancecheckout.models.transaction_user_interface_type import TransactionUserInterfaceType
from typing import Optional, Set
from typing_extensions import Self

class Transaction(BaseModel):
    """
    Transaction
    """ # noqa: E501
    parent: Optional[Transaction] = None
    total_settled_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount that was settled, in the transaction's currency.", alias="totalSettledAmount")
    device_session_identifier: Optional[Annotated[str, Field(min_length=10, strict=True, max_length=40)]] = Field(default=None, description="Allows to link the transaction to the data collected from the customer's device.", alias="deviceSessionIdentifier")
    processing_on: Optional[datetime] = Field(default=None, description="The date and time when the processing of the transaction was started.", alias="processingOn")
    invoice_merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the invoice.", alias="invoiceMerchantReference")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    confirmed_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction was created.", alias="confirmedOn")
    line_items: Optional[List[LineItem]] = Field(default=None, description="The line items purchased by the customer.", alias="lineItems")
    accept_language_header: Optional[StrictStr] = Field(default=None, description="The 'Accept Language' header of the customer's web browser.", alias="acceptLanguageHeader")
    java_enabled: Optional[StrictBool] = Field(default=None, description="Whether Java is enabled on the customer's web browser.", alias="javaEnabled")
    confirmed_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the transaction was confirmed by.", alias="confirmedBy")
    payment_connector_configuration: Optional[PaymentConnectorConfiguration] = Field(default=None, alias="paymentConnectorConfiguration")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[TransactionState] = None
    window_width: Optional[StrictStr] = Field(default=None, description="The window width of the customer's web browser.", alias="windowWidth")
    allowed_payment_method_configurations: Optional[List[StrictInt]] = Field(default=None, description="The payment method configurations that can be used to authorize the transaction.", alias="allowedPaymentMethodConfigurations")
    group: Optional[TransactionGroup] = None
    charge_retry_enabled: Optional[StrictBool] = Field(default=None, description="Whether the customer can make further payment attempts if the first one has failed. Default is true.", alias="chargeRetryEnabled")
    accept_header: Optional[StrictStr] = Field(default=None, description="The 'Accept' header of the customer's web browser.", alias="acceptHeader")
    user_agent_header: Optional[StrictStr] = Field(default=None, description="The 'User Agent' header of the customer's web browser.", alias="userAgentHeader")
    shipping_method: Optional[Annotated[str, Field(strict=True, max_length=200)]] = Field(default=None, description="The name of the shipping method used to ship the products.", alias="shippingMethod")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    success_url: Optional[Annotated[str, Field(min_length=9, strict=True, max_length=2000)]] = Field(default=None, description="The URL to redirect the customer back to after they successfully authenticated their payment.", alias="successUrl")
    time_zone: Optional[StrictStr] = Field(default=None, description="The customer's time zone, which affects how dates and times are formatted when communicating with the customer.", alias="timeZone")
    space_view_id: Optional[StrictInt] = Field(default=None, description="The ID of the space view this object is linked to.", alias="spaceViewId")
    user_failure_message: Optional[StrictStr] = Field(default=None, description="The message that can be displayed to the customer explaining why the transaction failed, in the customer's language.", alias="userFailureMessage")
    completion_behavior: Optional[TransactionCompletionBehavior] = Field(default=None, alias="completionBehavior")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    internet_protocol_address_country: Optional[StrictStr] = Field(default=None, description="The country determined from the IP address of the customer's device.", alias="internetProtocolAddressCountry")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    delivery_decision_made_on: Optional[datetime] = Field(default=None, description="This date and time when the decision was made as to whether the order should be shipped.", alias="deliveryDecisionMadeOn")
    authorization_environment: Optional[ChargeAttemptEnvironment] = Field(default=None, alias="authorizationEnvironment")
    auto_confirmation_enabled: Optional[StrictBool] = Field(default=None, description="Whether the transaction can be confirmed automatically or whether this must be done explicitly via the API. Default is true.", alias="autoConfirmationEnabled")
    failure_reason: Optional[FailureReason] = Field(default=None, alias="failureReason")
    total_applied_fees: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total of all fees charged, in the transaction's currency.", alias="totalAppliedFees")
    customers_presence: Optional[CustomersPresence] = Field(default=None, alias="customersPresence")
    failed_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction failed.", alias="failedOn")
    refunded_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount that was refunded, in the transaction's currency.", alias="refundedAmount")
    authorization_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The sum of all line item prices including taxes in the transaction's currency.", alias="authorizationAmount")
    screen_width: Optional[StrictStr] = Field(default=None, description="The screen width of the customer's web browser.", alias="screenWidth")
    environment_selection_strategy: Optional[TransactionEnvironmentSelectionStrategy] = Field(default=None, alias="environmentSelectionStrategy")
    customer_email_address: Optional[Annotated[str, Field(strict=True, max_length=254)]] = Field(default=None, description="The customer's email address.", alias="customerEmailAddress")
    window_height: Optional[StrictStr] = Field(default=None, description="The window height of the customer's web browser.", alias="windowHeight")
    tokenization_mode: Optional[TokenizationMode] = Field(default=None, alias="tokenizationMode")
    authorization_timeout_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction must be authorized, otherwise it will canceled.", alias="authorizationTimeoutOn")
    allowed_payment_method_brands: Optional[List[StrictInt]] = Field(default=None, description="The payment method brands that can be used to authorize the transaction.", alias="allowedPaymentMethodBrands")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    meta_data: Optional[Dict[str, StrictStr]] = Field(default=None, description="Allow to store additional information about the object.", alias="metaData")
    emails_disabled: Optional[StrictBool] = Field(default=None, description="Whether email sending is deactivated for the transaction. Default is false.", alias="emailsDisabled")
    user_interface_type: Optional[TransactionUserInterfaceType] = Field(default=None, alias="userInterfaceType")
    customer_id: Optional[StrictStr] = Field(default=None, description="The unique identifier of the customer in the external system.", alias="customerId")
    currency: Optional[StrictStr] = Field(default=None, description="The three-letter code (ISO 4217 format) of the transaction's currency.")
    merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the transaction.", alias="merchantReference")
    authorization_sales_channel: Optional[StrictInt] = Field(default=None, description="The sales channel through which the transaction was placed.", alias="authorizationSalesChannel")
    years_to_keep: Optional[StrictInt] = Field(default=None, description="The number of years the transaction is kept after its authorization.", alias="yearsToKeep")
    completed_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount that was completed, in the transaction's currency.", alias="completedAmount")
    screen_height: Optional[StrictStr] = Field(default=None, description="The screen height of the customer's web browser.", alias="screenHeight")
    internet_protocol_address: Optional[StrictStr] = Field(default=None, description="The IP address of the customer's device.", alias="internetProtocolAddress")
    terminal: Optional[PaymentTerminal] = None
    end_of_life: Optional[datetime] = Field(default=None, description="The date and time when the transaction reaches its end of live. No further actions can be carried out at this time.", alias="endOfLife")
    token: Optional[Token] = None
    environment: Optional[Environment] = None
    screen_color_depth: Optional[StrictStr] = Field(default=None, description="The screen color depth of the customer's web browser.", alias="screenColorDepth")
    created_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the transaction was created by.", alias="createdBy")
    completed_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction was completed.", alias="completedOn")
    completion_timeout_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction is completed automatically.", alias="completionTimeoutOn")
    shipping_address: Optional[Address] = Field(default=None, alias="shippingAddress")
    billing_address: Optional[Address] = Field(default=None, alias="billingAddress")
    authorized_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction was authorized.", alias="authorizedOn")
    failed_url: Optional[Annotated[str, Field(min_length=9, strict=True, max_length=2000)]] = Field(default=None, description="The URL to redirect the customer back to after they canceled or failed to authenticated their payment.", alias="failedUrl")
    __properties: ClassVar[List[str]] = ["parent", "totalSettledAmount", "deviceSessionIdentifier", "processingOn", "invoiceMerchantReference", "language", "confirmedOn", "lineItems", "acceptLanguageHeader", "javaEnabled", "confirmedBy", "paymentConnectorConfiguration", "id", "state", "windowWidth", "allowedPaymentMethodConfigurations", "group", "chargeRetryEnabled", "acceptHeader", "userAgentHeader", "shippingMethod", "plannedPurgeDate", "successUrl", "timeZone", "spaceViewId", "userFailureMessage", "completionBehavior", "version", "internetProtocolAddressCountry", "linkedSpaceId", "deliveryDecisionMadeOn", "authorizationEnvironment", "autoConfirmationEnabled", "failureReason", "totalAppliedFees", "customersPresence", "failedOn", "refundedAmount", "authorizationAmount", "screenWidth", "environmentSelectionStrategy", "customerEmailAddress", "windowHeight", "tokenizationMode", "authorizationTimeoutOn", "allowedPaymentMethodBrands", "createdOn", "metaData", "emailsDisabled", "userInterfaceType", "customerId", "currency", "merchantReference", "authorizationSalesChannel", "yearsToKeep", "completedAmount", "screenHeight", "internetProtocolAddress", "terminal", "endOfLife", "token", "environment", "screenColorDepth", "createdBy", "completedOn", "completionTimeoutOn", "shippingAddress", "billingAddress", "authorizedOn", "failedUrl"]

    @field_validator('device_session_identifier')
    def device_session_identifier_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"([a-zA-Z0-9_-])*", value):
            raise ValueError(r"must validate the regular expression /([a-zA-Z0-9_-])*/")
        return value

    @field_validator('invoice_merchant_reference')
    def invoice_merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    @field_validator('merchant_reference')
    def merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Transaction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "total_settled_amount",
            "device_session_identifier",
            "processing_on",
            "invoice_merchant_reference",
            "language",
            "confirmed_on",
            "line_items",
            "accept_language_header",
            "java_enabled",
            "confirmed_by",
            "id",
            "window_width",
            "allowed_payment_method_configurations",
            "charge_retry_enabled",
            "accept_header",
            "user_agent_header",
            "shipping_method",
            "planned_purge_date",
            "success_url",
            "time_zone",
            "space_view_id",
            "user_failure_message",
            "version",
            "internet_protocol_address_country",
            "linked_space_id",
            "delivery_decision_made_on",
            "auto_confirmation_enabled",
            "total_applied_fees",
            "failed_on",
            "refunded_amount",
            "authorization_amount",
            "screen_width",
            "customer_email_address",
            "window_height",
            "authorization_timeout_on",
            "allowed_payment_method_brands",
            "created_on",
            "meta_data",
            "emails_disabled",
            "customer_id",
            "currency",
            "merchant_reference",
            "authorization_sales_channel",
            "years_to_keep",
            "completed_amount",
            "screen_height",
            "internet_protocol_address",
            "end_of_life",
            "screen_color_depth",
            "created_by",
            "completed_on",
            "completion_timeout_on",
            "authorized_on",
            "failed_url",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of parent
        if self.parent:
            _dict['parent'] = self.parent.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of payment_connector_configuration
        if self.payment_connector_configuration:
            _dict['paymentConnectorConfiguration'] = self.payment_connector_configuration.to_dict()
        # override the default output from pydantic by calling `to_dict()` of group
        if self.group:
            _dict['group'] = self.group.to_dict()
        # override the default output from pydantic by calling `to_dict()` of failure_reason
        if self.failure_reason:
            _dict['failureReason'] = self.failure_reason.to_dict()
        # override the default output from pydantic by calling `to_dict()` of terminal
        if self.terminal:
            _dict['terminal'] = self.terminal.to_dict()
        # override the default output from pydantic by calling `to_dict()` of token
        if self.token:
            _dict['token'] = self.token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of shipping_address
        if self.shipping_address:
            _dict['shippingAddress'] = self.shipping_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of billing_address
        if self.billing_address:
            _dict['billingAddress'] = self.billing_address.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Transaction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "parent": Transaction.from_dict(obj["parent"]) if obj.get("parent") is not None else None,
            "totalSettledAmount": obj.get("totalSettledAmount"),
            "deviceSessionIdentifier": obj.get("deviceSessionIdentifier"),
            "processingOn": obj.get("processingOn"),
            "invoiceMerchantReference": obj.get("invoiceMerchantReference"),
            "language": obj.get("language"),
            "confirmedOn": obj.get("confirmedOn"),
            "lineItems": [LineItem.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "acceptLanguageHeader": obj.get("acceptLanguageHeader"),
            "javaEnabled": obj.get("javaEnabled"),
            "confirmedBy": obj.get("confirmedBy"),
            "paymentConnectorConfiguration": PaymentConnectorConfiguration.from_dict(obj["paymentConnectorConfiguration"]) if obj.get("paymentConnectorConfiguration") is not None else None,
            "id": obj.get("id"),
            "state": obj.get("state"),
            "windowWidth": obj.get("windowWidth"),
            "allowedPaymentMethodConfigurations": obj.get("allowedPaymentMethodConfigurations"),
            "group": TransactionGroup.from_dict(obj["group"]) if obj.get("group") is not None else None,
            "chargeRetryEnabled": obj.get("chargeRetryEnabled"),
            "acceptHeader": obj.get("acceptHeader"),
            "userAgentHeader": obj.get("userAgentHeader"),
            "shippingMethod": obj.get("shippingMethod"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "successUrl": obj.get("successUrl"),
            "timeZone": obj.get("timeZone"),
            "spaceViewId": obj.get("spaceViewId"),
            "userFailureMessage": obj.get("userFailureMessage"),
            "completionBehavior": obj.get("completionBehavior"),
            "version": obj.get("version"),
            "internetProtocolAddressCountry": obj.get("internetProtocolAddressCountry"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "deliveryDecisionMadeOn": obj.get("deliveryDecisionMadeOn"),
            "authorizationEnvironment": obj.get("authorizationEnvironment"),
            "autoConfirmationEnabled": obj.get("autoConfirmationEnabled"),
            "failureReason": FailureReason.from_dict(obj["failureReason"]) if obj.get("failureReason") is not None else None,
            "totalAppliedFees": obj.get("totalAppliedFees"),
            "customersPresence": obj.get("customersPresence"),
            "failedOn": obj.get("failedOn"),
            "refundedAmount": obj.get("refundedAmount"),
            "authorizationAmount": obj.get("authorizationAmount"),
            "screenWidth": obj.get("screenWidth"),
            "environmentSelectionStrategy": obj.get("environmentSelectionStrategy"),
            "customerEmailAddress": obj.get("customerEmailAddress"),
            "windowHeight": obj.get("windowHeight"),
            "tokenizationMode": obj.get("tokenizationMode"),
            "authorizationTimeoutOn": obj.get("authorizationTimeoutOn"),
            "allowedPaymentMethodBrands": obj.get("allowedPaymentMethodBrands"),
            "createdOn": obj.get("createdOn"),
            "metaData": obj.get("metaData"),
            "emailsDisabled": obj.get("emailsDisabled"),
            "userInterfaceType": obj.get("userInterfaceType"),
            "customerId": obj.get("customerId"),
            "currency": obj.get("currency"),
            "merchantReference": obj.get("merchantReference"),
            "authorizationSalesChannel": obj.get("authorizationSalesChannel"),
            "yearsToKeep": obj.get("yearsToKeep"),
            "completedAmount": obj.get("completedAmount"),
            "screenHeight": obj.get("screenHeight"),
            "internetProtocolAddress": obj.get("internetProtocolAddress"),
            "terminal": PaymentTerminal.from_dict(obj["terminal"]) if obj.get("terminal") is not None else None,
            "endOfLife": obj.get("endOfLife"),
            "token": Token.from_dict(obj["token"]) if obj.get("token") is not None else None,
            "environment": obj.get("environment"),
            "screenColorDepth": obj.get("screenColorDepth"),
            "createdBy": obj.get("createdBy"),
            "completedOn": obj.get("completedOn"),
            "completionTimeoutOn": obj.get("completionTimeoutOn"),
            "shippingAddress": Address.from_dict(obj["shippingAddress"]) if obj.get("shippingAddress") is not None else None,
            "billingAddress": Address.from_dict(obj["billingAddress"]) if obj.get("billingAddress") is not None else None,
            "authorizedOn": obj.get("authorizedOn"),
            "failedUrl": obj.get("failedUrl")
        })
        return _obj

# TODO: Rewrite to not use raise_errors
Transaction.model_rebuild(raise_errors=False)

